//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2019                                                //
//  IDS Imaging Development Systems GmbH                                     //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted     //
//  by IDS Imaging GmbH.                                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose  //
//  in calibrating and testing IDS manufactured cameras and software.        //
//                                                                           //
//===========================================================================//

#if !defined IDS_SHERPA
#define IDS_SHERPA

#include "stdafx.h"

#include <map>

#include <peak\peak.hpp>
#include <peak_ipl\peak_ipl.hpp>


#define TRIGGER_OFF                 0
#define TRIGGER_HI_LO               1
#define TRIGGER_LO_HI               2
#define TRIGGER_SOFTWARE            3 

#define GAIN_MASTER                 0
#define GAIN_MASTER_ANALOG          0
#define GAIN_MASTER_DIGITAL         1
#define GAIN_RED                    2
#define GAIN_GREEN                  3
#define GAIN_BLUE                   4 

#define DISPLAY_NORMAL              0
#define DISPLAY_FIT_TO_WINDOW       1

#define AUTO_EXPOSURE_OFF           0
#define AUTO_EXPOSURE_ONCE          1
#define AUTO_EXPOSURE_CONTINUOUS    2

#define AUTO_GAIN_OFF               0
#define AUTO_GAIN_ONCE              1
#define AUTO_GAIN_CONTINUOUS        2

#define USER_SET_0                  0
#define USER_SET_1                  1


struct CAMERA_INFORMATION
{
    std::string serialNumber;
    std::string manufacturer;
    std::string cameraType;
    std::string modelName;
    int maxSensorWidth;
    int maxSensorHeight;
    bool bayerPattern;
    std::string upperLeftBayerPixel;
    bool globalShutter;
};


class IdsSherpa
{

public:
	
    IdsSherpa(HWND hWnd = NULL);
	~IdsSherpa();

    bool SetWindowHandle(HWND hWnd);

    // Open/Close camera
	bool InitCamera(std::string serialNumber = "");
    bool ExitCamera();

    // Current number of connected cameras
    bool GetNumberOfCameras(int& number);

    // Source pixel format (Camera)
    bool SetColorModeSource(peak::ipl::PixelFormatName colorMode);
    bool GetColorModeSource(peak::ipl::PixelFormatName& colorMode);

    // Destination pixel format (Host rendering)
    bool SetColorModeDest(peak::ipl::PixelFormatName colorMode);
    bool GetColorModeDest(peak::ipl::PixelFormatName& colorMode);

    // Sets the destination pixel format and adapts the source pixel format to fit best to the destination
    bool SetColorMode(peak::ipl::PixelFormatName colorMode);
    bool GetColorMode(peak::ipl::PixelFormatName& colorMode);
    bool GetBitsPerPixel(peak::ipl::PixelFormatName colorMode, int& bitsPerPixel);

    // AOI
    bool GetImageAoiMin(int& x, int& y, int& width, int& height);
    bool GetImageAoiMax(int& x, int& y, int& width, int& height);
    bool GetImageAoi(int& x, int& y, int& width, int& height);
    bool SetImageAoi(int x, int y, int width, int height);

    // Information
    bool GetCameraInfo(CAMERA_INFORMATION& info);
    
    // Memory allocation
    bool AllocImageBuffersAndCreateSequence(int numberBuffers = 0);
    bool FreeImageBuffersAndClearSequence();

    // Initialization of the display window
    bool SetRenderType(int renderType);
    
    // Activation/Deactivation of capture events
    bool InitEvents();
    bool ExitEvents();

    // Start/Stop capture
	bool StartCapture();
	bool StopCapture();
    bool IsCapturing();

    // Framerate
    bool GetFramerateMin(double& min);
    bool GetFramerateMax(double& max);
    bool GetFramerate(double& framerate);
    bool SetFramerate(double framerate);

    // Exposure time
    bool GetExposureTimeMin_ms(double& min);
    bool GetExposureTimeMax_ms(double& max);
    bool GetExposureTime_ms(double& exposure);
    bool SetExposureTime_ms(double exposure);

    // Trigger
    bool GetTriggerMode(int& mode);
    bool SetTriggerMode(int mode);
    bool ForceTrigger();
    bool SetTriggerTimeout_ms(int timeout);
    bool GetTriggerDelayMin_us(int& min);
    bool GetTriggerDelayMax_us(int& max);
    bool GetTriggerDelayInc_us(int& inc);
    bool GetTriggerDelay_us(int& delay);
    bool SetTriggerDelay_us(int delay);

    // Gain
    bool GetGainMin(int type, double& min);
    bool GetGainMax(int type, double& max);
    bool GetGain(int type, double& gain);
    bool SetGain(int type, double gain);

    // Auto exposure
    bool GetAutoExposure(int& autoExposure);
    bool SetAutoExposure(int autoExposure);

    // Auto gain
    bool GetAutoGain(int& autoGain);
    bool SetAutoGain(int autoGain);

    // Parameters for the auto brightness (exposure + gain) 
    bool GetBrightnessAutoPercentileMin(double& min);
    bool GetBrightnessAutoPercentileMax(double& max);
    bool GetBrightnessAutoPercentile(double& value);
    bool SetBrightnessAutoPercentile(double value);

    bool GetBrightnessAutoTargetMin(int& min);
    bool GetBrightnessAutoTargetMax(int& max);
    bool GetBrightnessAutoTarget(int& value);
    bool SetBrightnessAutoTarget(int value);

    bool GetBrightnessAutoTargetToleranceMin(int& min);
    bool GetBrightnessAutoTargetToleranceMax(int& max);
    bool GetBrightnessAutoTargetTolerance(int& value);
    bool SetBrightnessAutoTargetTolerance(int value);

    // Parameter sets
    bool SaveParametersInCameraMemory(int userSet);
    bool LoadParametersFromCameraMemory(int userSet);

    bool GetSubsamplingFactor(int& x, int& y);
    bool SetSubsamplingFactor(int x, int y);

private:

	HWND m_hWnd;

	std::shared_ptr<peak::core::Device> m_device;
	std::shared_ptr<peak::core::DataStream> m_dataStream;
	std::shared_ptr<peak::core::NodeMap> m_nodemapRemoteDevice;

    peak::ipl::PixelFormatName m_pixelFormat;

    std::map<peak::ipl::PixelFormatName, std::string> m_mapPixelFormat;
    std::map<peak::ipl::PixelFormatName, peak::ipl::PixelFormatName> m_mapChoosePixelFormatColor;
    std::map<peak::ipl::PixelFormatName, peak::ipl::PixelFormatName> m_mapChoosePixelFormatMono;

	char* m_bmpInfo;
    int m_renderType;
    bool m_color;
	
	CWinThread* m_threadCapture;
	BOOL m_threadRunning;
	BOOL m_stopThread;

    // Internal functions
    bool InitDisplay();
    bool GainHelper(int type, int min_max_val, double& value);
    bool IsColorCamera();
	static UINT	CaptureWorker(LPVOID pParam);
};

#endif // #define IDS_SHERPA
